#ifndef _SEMANAGE_CONTEXT_INTERNAL_H_
#define _SEMANAGE_CONTEXT_INTERNAL_H_

#include <semanage/context_record.h>
#include "dso.h"

hidden_proto(semanage_context_clone)
    hidden_proto(semanage_context_free)
    hidden_proto(semanage_context_from_string)
    hidden_proto(semanage_context_to_string)
#endif
